from pathlib import Path

from pheval_exomiser.post_process.post_process_results_format import create_standardised_results
from pheval_exomiser.prepare.tool_specific_configuration_options import ExomiserConfigurations


def post_process_result_format(
    config: ExomiserConfigurations,
    raw_results_dir: Path,
    output_dir: Path,
    phenopacket_dir: Path,
    variant_analysis: bool,
    gene_analysis: bool,
    disease_analysis: bool,
    exomiser_version: str,
):
    """Standardise Exomiser json format to separated gene and variant results."""
    create_standardised_results(
        result_dir=raw_results_dir,
        output_dir=output_dir,
        phenopacket_dir=phenopacket_dir,
        sort_order=config.post_process.sort_order,
        score_name=config.post_process.score_name,
        gene_analysis=gene_analysis,
        disease_analysis=disease_analysis,
        variant_analysis=variant_analysis,
        exomiser_version=exomiser_version,
    )



import inspect
from pathlib import Path

from pheval_exomiser.constants import EXOMISER_DATA_DIRECTORY_TARGET_DOCKER
from pheval_exomiser.prepare.tool_specific_configuration_options import ExomiserConfigurations


class ExomiserConfigurationFileWriter:
    def __init__(self, input_dir: Path, configurations: ExomiserConfigurations):
        self.input_dir = input_dir
        self.configurations = configurations
        self.application_properties = open(input_dir.joinpath("application.properties"), "w")

    def write_remm_version(self) -> None:
        """Write the remm version to application.properties file."""
        if self.configurations.application_properties.remm_version is not None:
            self.application_properties.write(
                f"remm.version={self.configurations.application_properties.remm_version}\n"
            )

    def write_cadd_version(self) -> None:
        """Write the cadd version to application.properties file."""
        if self.configurations.application_properties.cadd_version is not None:
            self.application_properties.write(
                f"cadd.version={self.configurations.application_properties.cadd_version}\n"
            )

    def write_exomiser_data_directory(self) -> None:
        """Write the exomiser data directory to application.properties file."""
        if self.configurations.environment.lower() == "docker":
            self.application_properties.write(
                f"exomiser.data-directory={EXOMISER_DATA_DIRECTORY_TARGET_DOCKER}\n"
            )
        if self.configurations.environment.lower() == "local":
            self.application_properties.write(f"exomiser.data-directory={self.input_dir}\n")

    def write_exomiser_hg19_data_version(self) -> None:
        """Write the hg19 data version to application.properties file."""
        if self.configurations.application_properties.hg19_data_version is not None:
            self.application_properties.write(
                f"exomiser.hg19.data-version={self.configurations.application_properties.hg19_data_version}\n"
            )

    def write_exomiser_hg19_cadd_snv_path(self) -> None:
        """Write the hg19 cadd snv path to application.properties file."""
        if (
            self.configurations.application_properties.cadd_version is not None
            and self.configurations.application_properties.hg19_data_version is not None
        ):
            self.application_properties.write(
                "exomiser.hg19.cadd-snv-path="
                "${exomiser.data-directory}/cadd/${cadd.version}/hg19/"
                "whole_genome_SNVs.tsv.gz\n"
            )

    def write_exomiser_hg19_cadd_indel_path(self) -> None:
        """Write the hg19 cadd indel path to application.properties file."""
        if (
            self.configurations.application_properties.cadd_version is not None
            and self.configurations.application_properties.hg19_data_version is not None
        ):
            self.application_properties.write(
                "exomiser.hg19.cadd-in-del-path="
                "${exomiser.data-directory}/cadd/${cadd.version}/hg19/"
                "InDels.tsv.gz\n"
            )

    def write_exomiser_hg19_remm_path(self) -> None:
        """Write the hg19 remm path to application.properties file."""
        if (
            self.configurations.application_properties.remm_version is not None
            and self.configurations.application_properties.hg19_data_version is not None
        ):
            self.application_properties.write(
                "exomiser.hg19.remm-path="
                "${exomiser.data-directory}/remm/ReMM.v${remm.version}.hg19.tsv.gz\n"
            )

    def write_exomiser_hg19_local_frequency_path(self) -> None:
        """Write the hg19 local frequency path to application.properties file."""
        if self.configurations.application_properties.hg19_local_frequency_path is not None:
            self.application_properties.write(
                f"exomiser.hg19.local-frequency-path="
                f"${{exomiser.data-directory}}/local/"
                f"{self.configurations.application_properties.hg19_local_frequency_path}\n"
            )

    def write_exomiser_hg38_data_version(self) -> None:
        """Write the hg38 data version to application.properties file."""
        if self.configurations.application_properties.hg38_data_version is not None:
            self.application_properties.write(
                f"exomiser.hg38.data-version={self.configurations.application_properties.hg38_data_version}\n"
            )

    def write_exomiser_hg38_cadd_snv_path(self) -> None:
        """Write the hg38 cadd snv path to application.properties file."""
        if (
            self.configurations.application_properties.cadd_version is not None
            and self.configurations.application_properties.hg38_data_version is not None
        ):
            self.application_properties.write(
                "exomiser.hg38.cadd-snv-path="
                "${exomiser.data-directory}/cadd/${cadd.version}/hg38/"
                "whole_genome_SNVs.tsv.gz\n"
            )

    def write_exomiser_hg38_cadd_indel_path(self) -> None:
        """Write the hg38 cadd indel path to application.properties file."""
        if (
            self.configurations.application_properties.cadd_version is not None
            and self.configurations.application_properties.hg38_data_version is not None
        ):
            self.application_properties.write(
                "exomiser.hg38.cadd-in-del-path="
                "${exomiser.data-directory}/cadd/${cadd.version}/hg38/"
                "InDels.tsv.gz\n"
            )

    def write_exomiser_hg38_remm_path(self) -> None:
        """Write the hg38 remm path to application.properties file."""
        if (
            self.configurations.application_properties.remm_version is not None
            and self.configurations.application_properties.hg38_data_version is not None
        ):
            self.application_properties.write(
                "exomiser.hg38.remm-path="
                "${exomiser.data-directory}/remm/ReMM.v${remm.version}.hg38.tsv.gz\n"
            )

    def write_exomiser_hg38_local_frequency_path(self) -> None:
        """Write the hg38 local frequency path to application.properties file."""
        if self.configurations.application_properties.hg38_local_frequency_path is not None:
            self.application_properties.write(
                f"exomiser.hg38.local-frequency-path="
                f"${{exomiser.data-directory}}/local/"
                f"{self.configurations.application_properties.hg38_local_frequency_path}\n"
            )

    def write_exomiser_phenotype_data_version(self) -> None:
        """Write the phenotype data version to application.properties file."""
        self.application_properties.write(
            f"exomiser.phenotype.data-version={self.configurations.application_properties.phenotype_data_version}\n"
        )

    def write_hg19_white_list_path(self) -> None:
        """Write the hg19 whitelist path to application.properties file."""
        if self.configurations.application_properties.hg19_whitelist_path is not None:
            self.application_properties.write(
                f"exomiser.hg19.variant-white-list-path="
                f"{self.configurations.application_properties.hg19_whitelist_path}\n"
            )

    def write_hg38_white_list_path(self) -> None:
        """Write the hg38 whitelist path to application.properties file."""
        if self.configurations.application_properties.hg38_whitelist_path is not None:
            self.application_properties.write(
                f"exomiser.hg38.variant-white-list-path="
                f"{self.configurations.application_properties.hg38_whitelist_path}\n"
            )

    def write_cache_type(self):
        """Write the cache type to application.properties file."""
        if self.configurations.application_properties.cache_type is not None:
            self.application_properties.write(
                f"spring.cache.type=" f"{self.configurations.application_properties.cache_type}\n"
            )

    def write_cache_spec(self):
        """Write the cache spec to application.properties file."""
        if self.configurations.application_properties.cache_caffeine_spec is not None:
            self.application_properties.write(
                f"spring.cache.caffeine.spec=maximumSize="
                f"{self.configurations.application_properties.cache_caffeine_spec}\n"
            )

    def write_application_properties(self) -> None:
        """Write the application.properties file."""
        methods = inspect.getmembers(self, predicate=inspect.ismethod)
        for name, method in methods:
            if name != "write_application_properties" and name != "__init__":
                method()
        self.application_properties.close()



PHENOPACKET_TARGET_DIRECTORY_DOCKER = "/exomiser-testdata-phenopacket/"
RAW_RESULTS_TARGET_DIRECTORY_DOCKER = "/exomiser-results/"
OUTPUT_OPTIONS_TARGET_DIRECTORY_DOCKER = "/exomiser-testdata-output-options/"
VCF_TARGET_DIRECTORY_DOCKER = "/exomiser-testdata-vcf/"
EXOMISER_YAML_TARGET_DIRECTORY_DOCKER = "/exomiser-yaml-template/"
EXOMISER_DATA_DIRECTORY_TARGET_DOCKER = "/exomiser-data/"
INPUT_COMMANDS_TARGET_DIRECTORY_DOCKER = "/exomiser-batch-file/"
EXOMISER_CONFIG_TARGET_DIRECTORY_DOCKER = "/exomiser-config/"



import click

from .post_process.post_process_results_format import post_process_exomiser_results
from .prepare.create_batch_commands import prepare_exomiser_batch


@click.group()
def main():
    """Exomiser runner."""


main.add_command(prepare_exomiser_batch)
main.add_command(post_process_exomiser_results)

if __name__ == "__main__":
    main()



import tempfile
from dataclasses import dataclass
from pathlib import Path
from typing import List, Optional

import click
from packaging import version
from phenopackets import Family, Phenopacket
from pheval.prepare.custom_exceptions import MutuallyExclusiveOptionError
from pheval.utils.file_utils import all_files, files_with_suffix
from pheval.utils.phenopacket_utils import PhenopacketUtil, phenopacket_reader

from pheval_exomiser.constants import (
    EXOMISER_YAML_TARGET_DIRECTORY_DOCKER,
    OUTPUT_OPTIONS_TARGET_DIRECTORY_DOCKER,
    PHENOPACKET_TARGET_DIRECTORY_DOCKER,
    RAW_RESULTS_TARGET_DIRECTORY_DOCKER,
    VCF_TARGET_DIRECTORY_DOCKER,
)


@dataclass
class ExomiserCommandLineArguments:
    """Store command line arguments for each phenopacket to be run with Exomiser."""

    sample: Path
    analysis_yaml: Optional[Path] = None
    vcf_file: Optional[Path] = None
    vcf_assembly: Optional[str] = None
    raw_results_dir: Optional[Path] = None
    variant_analysis: Optional[bool] = None
    output_options_file: Optional[Path] = None
    output_formats: Optional[List[str]] = None


def get_all_files_from_output_opt_directory(output_options_dir: Path) -> List[Path] or None:
    """Obtain all output options files if directory is specified - otherwise returns none."""
    return None if output_options_dir is None else all_files(output_options_dir)


class CommandCreator:
    """Create a command for a phenopacket."""

    def __init__(
        self,
        environment: str,
        phenopacket_path: Path,
        phenopacket: Phenopacket or Family,
        variant_analysis: bool,
        output_options_dir_files: List[Path] or None,
        output_options_file: Path or None,
        raw_results_dir: Path or None,
        analysis_yaml: Path or None,
        output_formats: List[str] or None,
    ):
        self.environment = environment
        self.phenopacket_path = phenopacket_path
        self.phenopacket = phenopacket
        self.variant_analysis = variant_analysis
        self.output_options_dir_files = output_options_dir_files
        self.output_options_file = output_options_file
        self.results_dir = raw_results_dir
        self.analysis_yaml = analysis_yaml
        self.output_formats = output_formats

    def assign_output_options_file(self) -> Path or None:
        """Return the path of a single output option yaml if specified,
        otherwise return the best match from a directory."""
        if self.output_options_dir_files is None and self.output_options_file is None:
            return None
        else:
            return (
                self.output_options_file
                if self.output_options_dir_files is None
                else self.output_options_dir_files[0].parent.joinpath(
                    self.phenopacket_path.stem + "-output_options.yml"
                )
            )

    def add_phenotype_only_arguments(self) -> ExomiserCommandLineArguments:
        output_options_file = self.assign_output_options_file()
        if self.environment == "docker":
            return ExomiserCommandLineArguments(
                sample=Path(f"{PHENOPACKET_TARGET_DIRECTORY_DOCKER}{self.phenopacket_path.name}"),
                variant_analysis=self.variant_analysis,
                output_options_file=(
                    Path(f"{OUTPUT_OPTIONS_TARGET_DIRECTORY_DOCKER}{output_options_file.name}")
                    if output_options_file
                    else None
                ),
                raw_results_dir=Path(RAW_RESULTS_TARGET_DIRECTORY_DOCKER),
                output_formats=self.output_formats,
                analysis_yaml=self.analysis_yaml,
            )
        elif self.environment == "local":
            return ExomiserCommandLineArguments(
                sample=self.phenopacket_path,
                variant_analysis=self.variant_analysis,
                output_options_file=output_options_file,
                raw_results_dir=self.results_dir,
                output_formats=self.output_formats,
                analysis_yaml=self.analysis_yaml,
            )
        raise ValueError(f"Unknown environment: {self.environment}")

    def add_variant_analysis_arguments(self, vcf_dir: Path) -> ExomiserCommandLineArguments:
        if vcf_dir.exists():
            vcf_file_data = PhenopacketUtil(self.phenopacket).vcf_file_data(
                self.phenopacket_path, vcf_dir
            )
        else:
            vcf_file_data = next(
                file
                for file in self.phenopacket.files
                if file.file_attributes["fileFormat"] == "vcf"
            )
        output_options_file = self.assign_output_options_file()
        if self.environment == "local":
            return ExomiserCommandLineArguments(
                sample=self.phenopacket_path,
                vcf_file=Path(vcf_file_data.uri),
                vcf_assembly=vcf_file_data.file_attributes["genomeAssembly"],
                output_options_file=output_options_file,
                variant_analysis=self.variant_analysis,
                raw_results_dir=self.results_dir,
                analysis_yaml=self.analysis_yaml,
                output_formats=self.output_formats,
            )
        elif self.environment == "docker":
            return ExomiserCommandLineArguments(
                sample=Path(f"{PHENOPACKET_TARGET_DIRECTORY_DOCKER}{self.phenopacket_path.name}"),
                vcf_file=Path(f"{VCF_TARGET_DIRECTORY_DOCKER}{Path(vcf_file_data.uri).name}"),
                vcf_assembly=vcf_file_data.file_attributes["genomeAssembly"],
                output_options_file=(
                    Path(f"{OUTPUT_OPTIONS_TARGET_DIRECTORY_DOCKER}{output_options_file.name}")
                    if output_options_file
                    else None
                ),
                variant_analysis=self.variant_analysis,
                raw_results_dir=Path(RAW_RESULTS_TARGET_DIRECTORY_DOCKER),
                analysis_yaml=(
                    Path(f"{EXOMISER_YAML_TARGET_DIRECTORY_DOCKER}{self.analysis_yaml.name}")
                    if self.analysis_yaml
                    else None
                ),
                output_formats=self.output_formats,
            )
        raise ValueError(f"Unknown environment: {self.environment}")

    def add_command_line_arguments(self, vcf_dir: Path or None) -> ExomiserCommandLineArguments:
        """Return a dataclass of all the command line arguments corresponding to phenopacket sample."""
        return (
            self.add_variant_analysis_arguments(vcf_dir)
            if self.variant_analysis
            else self.add_phenotype_only_arguments()
        )


def create_command_arguments(
    environment: str,
    phenopacket_dir: Path,
    phenotype_only: bool,
    vcf_dir: Path,
    results_dir: Path or None,
    output_options_dir: Path or None = None,
    output_options_file: Path or None = None,
    analysis_yaml: Path or None = None,
    output_formats: List[str] or None = None,
) -> List[ExomiserCommandLineArguments]:
    """Return a list of Exomiser command line arguments for a directory of phenopackets."""
    phenopacket_paths = files_with_suffix(phenopacket_dir, ".json")
    commands = []
    output_option_dir_files = get_all_files_from_output_opt_directory(output_options_dir)
    for phenopacket_path in phenopacket_paths:
        phenopacket = phenopacket_reader(phenopacket_path)
        commands.append(
            CommandCreator(
                environment,
                phenopacket_path,
                phenopacket,
                phenotype_only,
                output_option_dir_files,
                output_options_file,
                results_dir,
                analysis_yaml,
                output_formats,
            ).add_command_line_arguments(vcf_dir)
        )
    return commands


class CommandsWriter:
    """Write a command to file."""

    def __init__(self, file: Path, variant_analysis: bool, exomiser_version: str):
        self.file = open(file, "w")
        self.variant_analysis = variant_analysis
        self.exomiser_version = exomiser_version

    def write_basic_analysis_command(self, command_arguments: ExomiserCommandLineArguments):
        """Write basic analysis command for Exomiser"""
        self.file.write(
            "--analysis "
            + str(command_arguments.analysis_yaml)
            + " --sample "
            + str(command_arguments.sample)
            + " --vcf "
            + str(command_arguments.vcf_file)
            + " --assembly "
            + command_arguments.vcf_assembly
            + " --output-filename "
            + f"{command_arguments.sample.stem}-exomiser"
        )

    def write_results_dir(self, command_arguments: ExomiserCommandLineArguments) -> None:
        """Write results directory for exomiser ≥13.2.0 to run."""
        (
            self.file.write(" --output-directory " + str(command_arguments.raw_results_dir))
            if command_arguments.raw_results_dir is not None
            else None
        )

    def write_output_options(self, command_arguments: ExomiserCommandLineArguments) -> None:
        """Write a command out for exomiser ≤13.1.0 to run - including output option file specified."""
        (
            self.file.write(" --output " + str(command_arguments.output_options_file))
            if command_arguments.output_options_file is not None
            else None
        )

    def write_output_format(self, command_arguments: ExomiserCommandLineArguments) -> None:
        """Write output formats for Exomiser raw result output."""
        (
            self.file.write(" --output-format " + ",".join(command_arguments.output_formats))
            if command_arguments.output_formats is not None
            else None
        )

    def write_analysis_command(self, command_arguments: ExomiserCommandLineArguments):
        try:
            self.write_basic_analysis_command(command_arguments)
            self.write_results_dir(command_arguments)
            self.write_output_options(command_arguments)
            self.write_output_format(command_arguments)
            self.file.write("\n")
        except IOError:
            print("Error writing ", self.file)

    def write_basic_phenotype_only_command(
        self, command_arguments: ExomiserCommandLineArguments
    ) -> None:
        """Write a phenotype-only command out for exomiser ≥13.2.0 to run."""
        if command_arguments.analysis_yaml is None:
            phenotype_only_statement = (
                " --preset phenotype-only"
                if version.parse(self.exomiser_version) < version.parse("15.0.0")
                else " --preset phenotype_only"
            )
        else:
            phenotype_only_statement = f" --analysis {str(command_arguments.analysis_yaml)}"
        self.file.write(
            "--sample "
            + str(command_arguments.sample)
            + " --output-directory "
            + str(command_arguments.raw_results_dir)
            + " --output-filename "
            + f"{Path(command_arguments.sample).stem}-exomiser"
            + phenotype_only_statement
        )

    def write_phenotype_only_command(self, command_arguments: ExomiserCommandLineArguments):
        try:
            self.write_basic_phenotype_only_command(command_arguments)
            self.write_output_options(command_arguments)
            self.write_output_format(command_arguments)
            self.file.write("\n")
        except IOError:
            print("Error writing ", self.file)

    def write_local_commands(self, command_arguments: ExomiserCommandLineArguments):
        (
            self.write_analysis_command(command_arguments)
            if self.variant_analysis
            else self.write_phenotype_only_command(command_arguments)
        )

    def close(self) -> None:
        """Close file."""
        try:
            self.file.close()
        except IOError:
            print("Error closing ", self.file)


class BatchFileWriter:
    """Write all the commands out to a batch file."""

    def __init__(
        self,
        command_arguments_list: List[ExomiserCommandLineArguments],
        variant_analysis: bool,
        output_dir: Path,
        batch_prefix: str,
        exomiser_version: str,
    ):
        self.command_arguments_list = command_arguments_list
        self.variant_analysis = variant_analysis
        self.output_dir = output_dir
        self.batch_prefix = batch_prefix
        self.exomiser_version = exomiser_version

    def write_commands(self, commands_writer: CommandsWriter) -> None:
        """Write command arguments to a file."""
        for command_arguments in self.command_arguments_list:
            commands_writer.write_local_commands(command_arguments)
        commands_writer.close()

    def write_temp_file(self) -> str:
        """Write commands out to a temporary file."""
        temp = tempfile.NamedTemporaryFile(delete=False)
        commands_writer = CommandsWriter(
            Path(temp.name), self.variant_analysis, self.exomiser_version
        )
        self.write_commands(commands_writer)
        return temp.name

    def write_all_commands(self) -> None:
        """Write all commands out to a single file."""
        commands_writer = CommandsWriter(
            Path(self.output_dir).joinpath(self.batch_prefix + "-exomiser-batch.txt"),
            self.variant_analysis,
            self.exomiser_version,
        )
        self.write_commands(commands_writer)

    def create_split_batch_files(self, max_jobs: int) -> None:
        """Split temp file into separate batch files, dependent on the number of max jobs allocated to each file."""
        temp_file_name = self.write_temp_file()
        lines_per_file, f_name = max_jobs, 0
        splitfile = None
        with open(temp_file_name) as tmp_file:
            for lineno, line in enumerate(tmp_file):
                if lineno % lines_per_file == 0:
                    f_name += 1
                    if splitfile:
                        splitfile.close()
                    split_filename = Path(self.output_dir).joinpath(
                        self.batch_prefix + "-exomiser-batch-{}.txt".format(f_name)
                    )
                    splitfile = open(split_filename, "w")
                splitfile.write(line)
            if splitfile:
                splitfile.close()
        tmp_file.close()
        Path(temp_file_name).unlink()


def create_batch_file(
    environment: str,
    analysis: Path,
    phenopacket_dir: Path,
    vcf_dir: Path,
    output_dir: Path,
    batch_prefix: str,
    max_jobs: int,
    variant_analysis: bool,
    results_dir: Path,
    exomiser_version: str,
    output_options_dir: Path = None,
    output_options_file: Path = None,
    output_formats: List[str] = None,
) -> None:
    """Create Exomiser batch files."""
    command_arguments = create_command_arguments(
        environment,
        phenopacket_dir,
        variant_analysis,
        vcf_dir,
        results_dir,
        output_options_dir,
        output_options_file,
        analysis,
        output_formats,
    )
    (
        BatchFileWriter(
            command_arguments, variant_analysis, output_dir, batch_prefix, exomiser_version
        ).write_all_commands()
        if max_jobs == 0
        else BatchFileWriter(
            command_arguments,
            variant_analysis,
            output_dir,
            batch_prefix,
            exomiser_version,
        ).create_split_batch_files(max_jobs)
    )


@click.command()
@click.option(
    "--environment",
    "-e",
    required=False,
    default="local",
    show_default=True,
    help="Environment to run commands.",
    type=click.Choice(["local", "docker"]),
)
@click.option(
    "--analysis-yaml",
    "-a",
    required=False,
    metavar="FILE",
    type=Path,
    help="Path to the analysis .yml file.",
)
@click.option(
    "--phenopacket-dir",
    "-p",
    required=True,
    metavar="PATH",
    type=Path,
    help="Path to phenopacket directory.",
)
@click.option(
    "--vcf-dir",
    "-v",
    metavar="PATH",
    type=Path,
    help="Path to VCF directory.",
)
@click.option(
    "--batch-prefix",
    "-b",
    required=False,
    metavar="TEXT",
    help="Prefix of generated batch files.",
    default="RUN",
    show_default=True,
)
@click.option(
    "--max-jobs",
    "-j",
    required=False,
    metavar="<int>",
    type=int,
    default=0,
    show_default=True,
    help="Number of jobs in each file.",
)
@click.option(
    "--variant-analysis",
    type=bool,
    default=False,
    is_flag=True,
    help="Run Exomiser with phenotype only preset - strongly recommended to run with versions 13.2.0 onwards.",
)
@click.option(
    "--output-dir",
    "-d",
    type=Path,
    required=False,
    help="Results directory for Exomiser results - compatible with versions 13.2.0 onwards.",
)
@click.option(
    "--results-dir",
    "-r",
    type=Path,
    required=False,
    help="Results directory for Exomiser results - compatible with versions 13.2.0 onwards.",
)
@click.option(
    "--exomiser-version",
    "-v",
    required=True,
    help="Exomiser version used to generate results.",
    default="15.0.0",
    show_default=True,
)
@click.option(
    "--output-options-dir",
    "-O",
    cls=MutuallyExclusiveOptionError,
    mutually_exclusive=["output_options_file"],
    required=False,
    metavar="PATH",
    type=Path,
    help="Path to the output options directory. ",
)
@click.option(
    "--output-options-file",
    "-o",
    cls=MutuallyExclusiveOptionError,
    mutually_exclusive=["output_options_dir"],
    required=False,
    metavar="FILE",
    type=Path,
    help="Path to the output options file. ",
)
@click.option(
    "--output-formats",
    "-f",
    multiple=True,
    help="One or more output formats (e.g., --output-format vcf --output-format json).",
)
def prepare_exomiser_batch(
    environment: str,
    analysis_yaml: Path,
    phenopacket_dir: Path,
    vcf_dir: Path,
    output_dir: Path,
    results_dir: Path,
    batch_prefix: str,
    max_jobs: int,
    variant_analysis: bool,
    exomiser_version: str,
    output_options_dir: Path = None,
    output_options_file: Path = None,
    output_formats: List[str] = None,
):
    """Generate Exomiser batch files."""
    Path(output_dir).joinpath("tool_input_commands").mkdir(exist_ok=True)
    create_batch_file(
        environment=environment,
        analysis=analysis_yaml,
        phenopacket_dir=phenopacket_dir,
        vcf_dir=vcf_dir,
        output_dir=output_dir,
        results_dir=results_dir,
        batch_prefix=batch_prefix,
        max_jobs=max_jobs,
        variant_analysis=variant_analysis,
        output_options_dir=output_options_dir,
        output_options_file=output_options_file,
        output_formats=list(output_formats),
        exomiser_version=exomiser_version,
    )



import uuid
from enum import Enum
from pathlib import Path

import click
import polars as pl
from packaging import version
from pheval.post_processing.post_processing import (
    SortOrder,
    generate_disease_result,
    generate_gene_result,
    generate_variant_result,
)
from pheval.utils.file_utils import files_with_suffix
from pheval.utils.logger import get_logger

EXOMISER_LT_15 = {"combinedScore", "priorityScore", "variantScore", "pValue"}
EXOMISER_GTE_15 = {"geneCombinedScore", "geneVariantScore", "pValue"}

ALL_SCORE_NAMES = sorted(EXOMISER_LT_15 | EXOMISER_GTE_15)

logger = get_logger()


def _allowed_score_names(exomiser_version: str) -> set[str]:
    return (
        EXOMISER_GTE_15
        if version.parse(exomiser_version) >= version.parse("15.0.0")
        else EXOMISER_LT_15
    )


class ModeOfInheritance(Enum):
    AUTOSOMAL_DOMINANT = 1
    AD = 1
    AUTOSOMAL_RECESSIVE = 2
    AR = 2
    X_DOMINANT = 1
    XD = 1
    X_RECESSIVE = 2
    XR = 2
    MITOCHONDRIAL = 3
    MT = 3


def check_score_name(score_name: str, version: str):
    """
    Validates the provided score name for compatibility with the specified Exomiser version.

    Args:
        score_name: str
            The name of the score to validate.
        version: str
            The Exomiser version for which the validation is performed.
    Raises:
        click.BadParameter: Raised if the provided score name is not valid,
        including the list of allowed score names in the error message.
    """
    allowed_score_names = _allowed_score_names(version)
    if score_name not in allowed_score_names:
        raise click.BadParameter(
            f"'{score_name}' is not valid for Exomiser {version}. "
            f"Allowed values: {', '.join(sorted(allowed_score_names))}.",
            param_hint="--score-name",
        )


def trim_exomiser_result_filename(exomiser_result_path: Path) -> Path:
    """Trim suffix appended to Exomiser JSON result path."""
    return Path(str(exomiser_result_path.name).replace("-exomiser", ""))


def extract_gene_results_from_json(
    exomiser_json_result: pl.DataFrame, score_name: str
) -> pl.DataFrame:
    score_expr = pl.col(score_name) if score_name in exomiser_json_result.columns else pl.lit(0.0)
    return exomiser_json_result.select(
        [
            pl.col("geneSymbol").alias("gene_symbol"),
            pl.col("geneIdentifier").struct.field("geneId").alias("gene_identifier"),
            score_expr.fill_null(0.0).alias("score"),
        ]
    ).drop_nulls()


def extract_gene_results_from_parquet(
    exomiser_parquet_result: pl.DataFrame, score_name: str, variant_analysis: bool
) -> pl.DataFrame:
    if variant_analysis:
        exomiser_parquet_result = exomiser_parquet_result.filter(
            pl.col("isContributingVariant") == True  # noqa
        )
    return exomiser_parquet_result.select(
        [
            pl.col("geneSymbol").alias("gene_symbol"),
            pl.col("ensemblGeneId").alias("gene_identifier"),
            pl.col(score_name).fill_null(0).alias("score"),
        ]
    ).unique(
        subset=["gene_symbol", "gene_identifier"],
        keep="first",
    )


def extract_disease_results_from_json(exomiser_json_result: pl.DataFrame) -> pl.DataFrame:
    try:
        return (
            exomiser_json_result.select(
                [
                    pl.col("priorityResults")
                    .struct.field("HIPHIVE_PRIORITY")
                    .struct.field("diseaseMatches")
                ]
            )
            .explode("diseaseMatches")
            .unnest("diseaseMatches")
            .unnest("model")
            .select([pl.col("diseaseId").alias("disease_identifier"), pl.col("score")])
            .drop_nulls()
        )
    except pl.exceptions.StructFieldNotFoundError:
        return pl.DataFrame(
            schema={
                "disease_identifier": pl.String,
                "score": pl.Float64,
            }
        )


def extract_disease_results_from_parquet(exomiser_parquet_result: pl.DataFrame) -> pl.DataFrame:
    return (
        exomiser_parquet_result.select(pl.col("diseaseMatches"))
        .explode("diseaseMatches")
        .select(
            pl.col("diseaseMatches").struct.field("diseaseId").alias("disease_identifier"),
            pl.col("diseaseMatches").struct.field("score").alias("score"),
        )
        .drop_nulls()
    )


def extract_variant_results_from_json(
    exomiser_json_result: pl.DataFrame, score_name: str
) -> pl.DataFrame:
    return (
        exomiser_json_result.filter(pl.col("geneScores").is_not_null())
        .select([pl.col("geneScores"), pl.col(score_name).alias("score"), pl.col("geneSymbol")])
        .explode("geneScores")
        .unnest("geneScores")
        .filter(pl.col("contributingVariants").is_not_null())
        .explode("contributingVariants")
        .with_columns(
            [
                pl.col("contributingVariants").struct.field("contigName").alias("chrom"),
                pl.col("contributingVariants").struct.field("start"),
                pl.col("contributingVariants").struct.field("end"),
                pl.col("contributingVariants").struct.field("ref"),
                pl.col("contributingVariants")
                .struct.field("alt")
                .fill_null("")
                .str.strip_chars("<>")
                .alias("alt"),
                pl.col("modeOfInheritance")
                .map_elements(lambda moi: ModeOfInheritance[moi].value, return_dtype=pl.Int8)
                .alias("moi_enum"),
            ]
        )
        .with_columns(
            [
                (pl.col("moi_enum") == 2).alias("is_recessive"),
                pl.when(pl.col("moi_enum") == 2)
                .then(
                    pl.format(
                        "recessive|{}|{}|{}",
                        pl.col("geneSymbol"),
                        pl.col("score"),
                        pl.col("moi_enum"),
                    )
                )
                .otherwise(
                    pl.format(
                        "dominant|{}|{}|{}|{}|{}|{}",
                        pl.col("chrom"),
                        pl.col("start"),
                        pl.col("end"),
                        pl.col("ref"),
                        pl.col("alt"),
                        pl.col("score"),
                    )
                )
                .alias("group_key"),
            ]
        )
        .with_columns(
            [
                pl.col("group_key")
                .rank("dense")
                .cast(pl.UInt32)
                .map_elements(
                    lambda i: str(uuid.uuid5(uuid.NAMESPACE_DNS, str(i))), return_dtype=pl.String
                )
                .alias("grouping_id")
            ]
        )
        .select(
            ["chrom", "start", "end", "ref", "alt", "score", "modeOfInheritance", "grouping_id"]
        )
    )


def extract_variant_results_from_parquet(
    exomiser_parquet_result: pl.DataFrame, score_name: str
) -> pl.DataFrame:
    contributing_variant_only = exomiser_parquet_result.filter(
        pl.col("isContributingVariant") == True  # noqa
    )
    return (
        contributing_variant_only.select(
            [
                pl.col("geneSymbol"),
                pl.col("contigName").alias("chrom").cast(pl.String),
                pl.col("start").cast(pl.Int64),
                pl.col("end").cast(pl.Int64),
                pl.col("ref"),
                pl.col("alt"),
                pl.col(score_name).alias("score"),
                pl.col("moi")
                .map_elements(lambda moi: ModeOfInheritance[moi].value, return_dtype=pl.Int8)
                .alias("moi_enum"),
            ]
        )
        .with_columns(
            [
                (pl.col("moi_enum") == 2).alias("is_recessive"),
                pl.when(pl.col("moi_enum") == 2)
                .then(
                    pl.format(
                        "recessive|{}|{}|{}",
                        pl.col("geneSymbol"),
                        pl.col("score"),
                        pl.col("moi_enum"),
                    )
                )
                .otherwise(
                    pl.format(
                        "dominant|{}|{}|{}|{}|{}|{}",
                        pl.col("chrom"),
                        pl.col("start"),
                        pl.col("end"),
                        pl.col("ref"),
                        pl.col("alt"),
                        pl.col("score"),
                    )
                )
                .alias("group_key"),
            ]
        )
        .with_columns(
            [
                pl.col("group_key")
                .rank("dense")
                .cast(pl.UInt32)
                .map_elements(
                    lambda i: str(uuid.uuid5(uuid.NAMESPACE_DNS, str(i))), return_dtype=pl.String
                )
                .alias("grouping_id")
            ]
        )
    )


def create_standardised_results(
    result_dir: Path,
    output_dir: Path,
    phenopacket_dir: Path,
    score_name: str,
    sort_order: str,
    gene_analysis: bool,
    disease_analysis: bool,
    variant_analysis: bool,
    exomiser_version: str,
):
    sort_order = SortOrder.ASCENDING if sort_order.lower() == "ascending" else SortOrder.DESCENDING
    use_parquet = True if version.parse(exomiser_version) >= version.parse("15.0.0") else False
    result_files = (
        files_with_suffix(result_dir, ".parquet")
        if use_parquet
        else files_with_suffix(result_dir, ".json")
    )
    for exomiser_result_path in result_files:
        try:
            if use_parquet:
                exomiser_result = pl.read_parquet(exomiser_result_path)
            else:
                exomiser_result = pl.read_json(exomiser_result_path, infer_schema_length=None)
            if gene_analysis:
                gene_results = (
                    extract_gene_results_from_parquet(exomiser_result, score_name, variant_analysis)
                    if use_parquet
                    else extract_gene_results_from_json(exomiser_result, score_name)
                )
                generate_gene_result(
                    results=gene_results,
                    sort_order=sort_order,
                    output_dir=output_dir,
                    result_path=trim_exomiser_result_filename(exomiser_result_path),
                    phenopacket_dir=phenopacket_dir,
                )
            if disease_analysis:
                disease_results = (
                    extract_disease_results_from_parquet(exomiser_result)
                    if use_parquet
                    else extract_disease_results_from_json(exomiser_result)
                )
                generate_disease_result(
                    results=disease_results,
                    sort_order=sort_order,
                    output_dir=output_dir,
                    result_path=trim_exomiser_result_filename(exomiser_result_path),
                    phenopacket_dir=phenopacket_dir,
                )
            if variant_analysis:
                variant_results = (
                    extract_variant_results_from_parquet(exomiser_result, score_name)
                    if use_parquet
                    else extract_variant_results_from_json(exomiser_result, score_name)
                )
                generate_variant_result(
                    results=variant_results,
                    sort_order=sort_order,
                    output_dir=output_dir,
                    result_path=trim_exomiser_result_filename(exomiser_result_path),
                    phenopacket_dir=phenopacket_dir,
                )
        except Exception:
            logger.exception("Failed processing Exomiser result file: %s", exomiser_result_path)
            raise


@click.command()
@click.option(
    "--output-dir",
    "-o",
    required=True,
    metavar="PATH",
    help="Output directory for standardised results.",
    type=Path,
)
@click.option(
    "--results-dir",
    "-R",
    required=True,
    metavar="DIRECTORY",
    help="Full path to Exomiser results directory to be standardised.",
    type=Path,
)
@click.option(
    "--phenopacket-dir",
    "-p",
    required=True,
    metavar="DIRECTORY",
    help="Full path to phenopacket dir used to generate the raw results.",
    type=Path,
)
@click.option(
    "--score-name",
    "-s",
    type=click.Choice(ALL_SCORE_NAMES),
    help="Score column to extract (valid values depend on Exomiser version).",
    required=True,
)
@click.option(
    "--sort-order",
    "-so",
    required=True,
    help="Ordering of results for ranking.",
    type=click.Choice(["ascending", "descending"]),
    default="descending",
    show_default=True,
)
@click.option(
    "--gene-analysis/--no-gene-analysis",
    type=bool,
    default=False,
    help="Specify whether to create PhEval gene results.",
)
@click.option(
    "--variant-analysis/--no-variant-analysis",
    type=bool,
    default=False,
    help="Specify whether to create PhEval variant results.",
)
@click.option(
    "--disease-analysis/--no-disease-analysis",
    type=bool,
    default=False,
    help="Specify whether to create PhEval disease results.",
)
@click.option(
    "--version",
    "-v",
    required=True,
    help="Exomiser version used to generate results.",
    default="15.0.0",
    show_default=True,
)
def post_process_exomiser_results(
    output_dir: Path,
    results_dir: Path,
    phenopacket_dir: Path,
    score_name: str,
    sort_order: str,
    gene_analysis: bool,
    variant_analysis: bool,
    disease_analysis: bool,
    version: str,
):
    """Post-process Exomiser json results into PhEval gene and variant outputs."""
    (
        output_dir.joinpath("pheval_gene_results").mkdir(parents=True, exist_ok=True)
        if gene_analysis
        else None
    )
    (
        output_dir.joinpath("pheval_variant_results").mkdir(parents=True, exist_ok=True)
        if variant_analysis
        else None
    )
    (
        output_dir.joinpath("pheval_disease_results").mkdir(parents=True, exist_ok=True)
        if disease_analysis
        else None
    )
    check_score_name(score_name, version)
    create_standardised_results(
        result_dir=results_dir,
        output_dir=output_dir,
        phenopacket_dir=phenopacket_dir,
        score_name=score_name,
        sort_order=sort_order,
        variant_analysis=variant_analysis,
        gene_analysis=gene_analysis,
        disease_analysis=disease_analysis,
        exomiser_version=version,
    )



import os
import subprocess
from dataclasses import dataclass
from pathlib import Path

import docker
from packaging import version
from pheval.utils.file_utils import all_files

from pheval_exomiser.constants import (
    EXOMISER_CONFIG_TARGET_DIRECTORY_DOCKER,
    EXOMISER_DATA_DIRECTORY_TARGET_DOCKER,
    EXOMISER_YAML_TARGET_DIRECTORY_DOCKER,
    INPUT_COMMANDS_TARGET_DIRECTORY_DOCKER,
    PHENOPACKET_TARGET_DIRECTORY_DOCKER,
    RAW_RESULTS_TARGET_DIRECTORY_DOCKER,
    VCF_TARGET_DIRECTORY_DOCKER,
)
from pheval_exomiser.prepare.create_batch_commands import create_batch_file
from pheval_exomiser.prepare.tool_specific_configuration_options import ExomiserConfigurations


def prepare_batch_files(
    input_dir: Path,
    testdata_dir: Path,
    config: ExomiserConfigurations,
    tool_input_commands_dir: Path,
    raw_results_dir: Path,
    variant_analysis: bool,
    exomiser_version: str,
) -> None:
    """Prepare the exomiser batch files"""
    print("...preparing batch files...")
    vcf_dir_name = Path(testdata_dir).joinpath("vcf")
    if version.parse(exomiser_version) >= version.parse("15.0.0"):
        if "PARQUET" not in config.output_formats:
            config.output_formats.append("PARQUET")
    elif version.parse(exomiser_version) < version.parse("15.0.0"):
        if "JSON" not in config.output_formats:
            config.output_formats.append("JSON")
    create_batch_file(
        environment=config.environment,
        analysis=(
            input_dir.joinpath(config.analysis_configuration_file)
            if config.analysis_configuration_file
            else None
        ),
        phenopacket_dir=Path(testdata_dir).joinpath("phenopackets"),
        vcf_dir=vcf_dir_name if variant_analysis else None,
        output_dir=tool_input_commands_dir,
        batch_prefix=Path(testdata_dir).name,
        max_jobs=config.max_jobs,
        output_options_file=None,
        output_options_dir=None,
        results_dir=raw_results_dir,
        variant_analysis=variant_analysis,
        output_formats=config.output_formats,
        exomiser_version=exomiser_version,
    )


@dataclass
class BasicDockerMountsForExomiser:
    """Paths to mount for running Exomiser with docker."""

    phenopacket_test_data: str
    vcf_test_data: str
    exomiser_yaml: str
    tool_input_commands_path: str
    exomiser_data_dir: str
    raw_results_dir: Path
    exomiser_application_properties: Path


def mount_docker(
    input_dir: Path,
    testdata_dir: Path,
    tool_input_commands_dir: Path,
    raw_results_dir: Path,
    variant_analysis: bool,
) -> BasicDockerMountsForExomiser:
    """Create docker mounts for paths required for running Exomiser."""
    phenopacket_test_data = (
        f"{Path(testdata_dir).joinpath('phenopackets')}"
        f"{os.sep}:{PHENOPACKET_TARGET_DIRECTORY_DOCKER}"
    )
    vcf_test_data = (
        (f"{Path(testdata_dir).joinpath('vcf')}" f"{os.sep}:{VCF_TARGET_DIRECTORY_DOCKER}")
        if variant_analysis
        else None
    )
    exomiser_yaml = f"{input_dir}:{EXOMISER_YAML_TARGET_DIRECTORY_DOCKER}"
    batch_file_path = f"{tool_input_commands_dir}/:{INPUT_COMMANDS_TARGET_DIRECTORY_DOCKER}"
    exomiser_data_dir = f"{input_dir}{os.sep}:{EXOMISER_DATA_DIRECTORY_TARGET_DOCKER}"
    results_dir = f"{raw_results_dir}/:{RAW_RESULTS_TARGET_DIRECTORY_DOCKER}"
    exomiser_config = f"{input_dir}{os.sep}" f":{EXOMISER_CONFIG_TARGET_DIRECTORY_DOCKER}"
    return BasicDockerMountsForExomiser(
        phenopacket_test_data=phenopacket_test_data,
        vcf_test_data=vcf_test_data,
        exomiser_yaml=exomiser_yaml,
        tool_input_commands_path=batch_file_path,
        exomiser_data_dir=exomiser_data_dir,
        exomiser_application_properties=exomiser_config,
        raw_results_dir=results_dir,
    )


def run_exomiser_local(
    input_dir: Path,
    testdata_dir: Path,
    config: ExomiserConfigurations,
    output_dir: Path,
    tool_input_commands_dir: Path,
    exomiser_version: str,
) -> None:
    """Run Exomiser locally."""
    print("...running exomiser...")
    os.chdir(output_dir)
    batch_files = [
        file
        for file in all_files(tool_input_commands_dir)
        if file.name.startswith(Path(testdata_dir).name)
    ]
    exomiser_jar_file = [
        filename
        for filename in all_files(input_dir.joinpath(config.exomiser_software_directory))
        if filename.name.endswith(".jar")
    ][0]
    exomiser_jar_file_path = config.exomiser_software_directory.joinpath(exomiser_jar_file)
    for file in batch_files:
        if version.parse(exomiser_version) < version.parse("15.0.0"):
            subprocess.run(
                [
                    "java",
                    "-Xmx4g",
                    "-jar",
                    exomiser_jar_file_path,
                    "--batch",
                    file,
                    f"--spring.config.location={Path(input_dir).joinpath('application.properties')}",
                ],
                shell=False,
            )
        elif version.parse(exomiser_version) >= version.parse("15.0.0"):
            subprocess.run(
                [
                    "java",
                    "-Xmx4g",
                    f"-Dspring.config.location={str(Path(input_dir).joinpath('application.properties'))}",
                    "-jar",
                    exomiser_jar_file_path,
                    "batch",
                    file,
                ],
                shell=False,
            )
    if version.parse(exomiser_version) < version.parse("13.1.0"):
        os.rename(
            f"{output_dir}/results",
            output_dir.joinpath("raw_results"),
        )


def create_docker_run_command(batch_file: Path) -> [str]:
    """Creates docker run command."""
    return [
        "--batch",
        f"{INPUT_COMMANDS_TARGET_DIRECTORY_DOCKER}" + batch_file.name,
        f"--spring.config.location={EXOMISER_CONFIG_TARGET_DIRECTORY_DOCKER}application.properties",
    ]


def run_exomiser_docker(
    input_dir: Path,
    testdata_dir: Path,
    tool_input_commands_dir: Path,
    raw_results_dir: Path,
    exomiser_version: str,
    variant_analysis: bool,
):
    """Run Exomiser with docker."""
    print("...running exomiser...")
    client = docker.from_env()
    batch_files = [
        file
        for file in all_files(tool_input_commands_dir)
        if file.name.startswith(Path(testdata_dir).name)
    ]
    for file in batch_files:
        docker_command = create_docker_run_command(file)
        docker_mounts = mount_docker(
            input_dir, testdata_dir, tool_input_commands_dir, raw_results_dir, variant_analysis
        )
        vol = [
            docker_mounts.vcf_test_data,
            docker_mounts.phenopacket_test_data,
            docker_mounts.exomiser_data_dir,
            docker_mounts.exomiser_yaml,
            docker_mounts.tool_input_commands_path,
            docker_mounts.exomiser_application_properties,
            docker_mounts.raw_results_dir,
        ]
        container = client.containers.run(
            f"exomiser/exomiser-cli:{exomiser_version}",
            " ".join(docker_command),
            volumes=[x for x in vol if x is not None],
            detach=True,
        )
        for line in container.logs(stream=True):
            print(line.strip())
        break


def run_exomiser(
    input_dir: Path,
    testdata_dir,
    config: ExomiserConfigurations,
    output_dir: Path,
    tool_input_commands_dir: Path,
    raw_results_dir: Path,
    exomiser_version: str,
    variant_analysis: bool,
):
    """Run Exomiser with specified environment."""
    (
        run_exomiser_local(
            input_dir, testdata_dir, config, output_dir, tool_input_commands_dir, exomiser_version
        )
        if config.environment == "local"
        else run_exomiser_docker(
            input_dir,
            testdata_dir,
            tool_input_commands_dir,
            raw_results_dir,
            exomiser_version,
            variant_analysis,
        )
    )



from pathlib import Path
from typing import List, Optional, Union

from pydantic import BaseModel, Field


class ApplicationProperties(BaseModel):
    """
    Class for defining the application.properties configurations.
    Args:
        remm_version (str): Version of the REMM database
        cadd_version (str): Version of the CADD database
        hg19_data_version (str): Data version of the hg19 Exomiser data
        hg19_local_frequency_path (Path): The file name of the hg19 local frequency file
        hg19_whitelist_path (Path): The file name of the hg19 whitelist.
        hg38_data_version (str): Data version of the hg38 Exomiser data
        hg38_local_frequency_path (Path): The file name of the hg38 local frequency file
        hg38_whitelist_path (Path): The file name of the hg38 whitelist.
        phenotype_data_version (str): Data version of the Exomiser phenotype data
        cache_caffeine_spec (int): Cache limit
    """

    remm_version: Optional[str] = Field(None)
    cadd_version: Optional[str] = Field(None)
    hg19_data_version: Optional[str] = Field(None)
    hg19_local_frequency_path: Optional[Path] = Field(None)
    hg19_whitelist_path: Optional[Path] = Field(None)
    hg38_data_version: Optional[str] = Field(None)
    hg38_local_frequency_path: Optional[Path] = Field(None)
    hg38_whitelist_path: Optional[Path] = Field(None)
    phenotype_data_version: Optional[str] = Field(None)
    cache_type: Optional[str] = Field(None)
    cache_caffeine_spec: Optional[int] = Field(None)


class PostProcessing(BaseModel):
    """
    Class for defining the post-processing configurations.
    Args:
        score_name (str): Name of score to extract from results.
        sort_order (str): Order to sort results
    """

    score_name: str = Field(...)
    sort_order: str = Field(...)


class ExomiserConfigurations(BaseModel):
    """
    Class for defining the Exomiser configurations in tool_specific_configurations field,
    within the input_dir config.yaml
    Args:
        environment (str): Environment to run Exomiser, i.e., local/docker
        exomiser_software_directory (Path): Directory name for Exomiser software directory
        analysis_configuration_file (Path): The file name of the analysis configuration file located in the input_dir
        max_jobs (int): Maximum number of jobs to run in a batch
        application_properties (ApplicationProperties): application.properties configurations
        output_formats: List(str): List of raw output formats.
        post_process (PostProcessing): Post-processing configurations
    """

    environment: str = Field(...)
    exomiser_software_directory: Path = Field(...)
    analysis_configuration_file: Union[Path | None] = Field(...)
    max_jobs: int = Field(...)
    application_properties: ApplicationProperties = Field(...)
    output_formats: Optional[List[str]] = Field(None)
    post_process: PostProcessing = Field(...)



"""Exomiser Runner"""

from dataclasses import dataclass
from pathlib import Path

from pheval.runners.runner import PhEvalRunner

from pheval_exomiser.post_process.post_process import post_process_result_format
from pheval_exomiser.prepare.tool_specific_configuration_options import ExomiserConfigurations
from pheval_exomiser.prepare.write_application_properties import ExomiserConfigurationFileWriter
from pheval_exomiser.run.run import prepare_batch_files, run_exomiser


@dataclass
class ExomiserPhEvalRunner(PhEvalRunner):
    """_summary_"""

    input_dir: Path
    testdata_dir: Path
    tmp_dir: Path
    output_dir: Path
    config_file: Path
    version: str

    def prepare(self):
        """prepare"""
        print("preparing")
        ExomiserConfigurationFileWriter(
            input_dir=self.input_dir,
            configurations=ExomiserConfigurations.parse_obj(
                self.input_dir_config.tool_specific_configuration_options
            ),
        ).write_application_properties()

    def run(self):
        """run"""
        print("running with exomiser")
        config = ExomiserConfigurations.parse_obj(
            self.input_dir_config.tool_specific_configuration_options
        )
        prepare_batch_files(
            input_dir=self.input_dir,
            config=config,
            testdata_dir=self.testdata_dir,
            tool_input_commands_dir=self.tool_input_commands_dir,
            raw_results_dir=self.raw_results_dir,
            variant_analysis=self.input_dir_config.variant_analysis,
            exomiser_version=self.version,
        )
        run_exomiser(
            input_dir=self.input_dir,
            testdata_dir=self.testdata_dir,
            config=config,
            output_dir=self.output_dir,
            tool_input_commands_dir=self.tool_input_commands_dir,
            raw_results_dir=self.raw_results_dir,
            exomiser_version=self.version,
            variant_analysis=self.input_dir_config.variant_analysis,
        )

    def post_process(self):
        """post_process"""
        print("post processing")
        config = ExomiserConfigurations.parse_obj(
            self.input_dir_config.tool_specific_configuration_options
        )
        post_process_result_format(
            config=config,
            raw_results_dir=self.raw_results_dir,
            output_dir=self.output_dir,
            phenopacket_dir=self.testdata_dir.joinpath("phenopackets"),
            variant_analysis=self.input_dir_config.variant_analysis,
            gene_analysis=self.input_dir_config.gene_analysis,
            disease_analysis=self.input_dir_config.disease_analysis,
            exomiser_version=self.version,
        )
